`timescale 1ns/1ps

module lo_controlLogic(gateFe, dacStrobe, dacData, dac8staticData,
    countersPulse, clearCountersB, loadRam, initDacsB, calibTrain, invalidaddr,
    shiftClkEnable, readHeader, tokenStart, tokenClk,
    clk, command, padClrB, id);

    output [1:0] gateFe;
    output [11:0] dacStrobe;
    output [7:0] dacData;
    output [5:0] dac8staticData;
    output countersPulse, clearCountersB, loadRam, initDacsB, calibTrain;
    output shiftClkEnable, readHeader, tokenStart, tokenClk;
    output invalidaddr;
    input clk, command, padClrB;
    input [1:0] id;

    //Synchronic Reset module
    wire clrB;
    lo_4clkReset synchronicReset(.out(clrB), .clk(clk), .in(padClrB));

    //Reset during Initialize command
    wire int_clrB, resetLogic;
    assign int_clrB = clrB & ~resetLogic;

    wire initialize, loadandclear, loadram, readdata, counterspulse,
        setgate, calibtrain, loaddac;

    //DACs
    lo_12shiftRegAddr dacsMem(.data(dacData), .strobe(dacStrobe), .clk(clk),
        .clrB(int_clrB), .in(command), .shift(loaddac));
    lo_parMem configRegister(.out(dac8staticData), .clk(clk), .clrB(int_clrB),
        .strobe(dacStrobe[8]), .in(dacData[5:0]));
    defparam configRegister.width = 6;

    //Gates
    //Gates are closed during loadRam and clearCounters
    wire loadRamInProgress, clearCountersInProgress;
    wire [1:0] gateStatus;
    assign gateFe[1] = gateStatus[1] & ~(loadRamInProgress 
        | clearCountersInProgress);
    assign gateFe[0] = gateStatus[0] & ~(loadRamInProgress 
        | clearCountersInProgress);
    lo_2shiftRegMem gatesMem(.out(gateStatus[1:0]), .clk(clk), .clrB(int_clrB),
        .in(command), .shift(setgate));

    //Load Counters
    //Closes gates loadRam and restore gates to its previous state
    wire loadRamFinish;
    lo_loadOrClear loadRamMod(.go(loadRam), .finish(loadRamFinish),
        .disableGates(loadRamInProgress), .clk(clk), .clrB(int_clrB),
        .start(loadram|loadandclear));

    //Clear Counters
    //Fire loadRam and then clear counters. Of course gates are also
    //closed during counters clearing.
    wire clearCounters, loadAndClearInProgress, clearCountersFinish;
    assign clearCountersB = ~clearCounters & clrB; 
    lo_rs clearCountersRS(.out(loadAndClearInProgress), .clk(clk),
        .clrB(int_clrB), .set(loadandclear), .reset(clearCountersFinish));
    lo_loadOrClear clearCountersMod(.go(clearCounters), 
        .finish(clearCountersFinish), .disableGates(clearCountersInProgress),
        .clk(clk), .clrB(int_clrB), 
        .start( (loadRamFinish & loadAndClearInProgress)|initialize ));
   
    //Initialize
    //First clear counters and DACs then reset control logic.
    //The only module that is NOT reset is initializeDel1 below
    wire initializeInProgress;
    assign initDacsB = ~(initializeInProgress & clearCounters) & clrB;
    lo_rs initalizeRS(.out(initializeInProgress), .clk(clk),
        .clrB(int_clrB), .set(initialize), .reset(clearCountersFinish));
    lo_del1clk initializeDel1(.out(resetLogic), .clk(clk), .clrB(clrB),
        .in(initializeInProgress & clearCountersFinish));
    

    //Send impulse to counters
    lo_3clkImp counterPulse(.out(countersPulse), .clk(clk), .clrB(int_clrB),
        .start(counterspulse));
    
    //Generate calibration train
    wire [1:0] calibParams;
    wire calibTrainStart;
    lo_2shiftRegStrobe calibParamsMem(.out(calibParams), 
        .finish(calibTrainStart),.clk(clk), .clrB(int_clrB), .in(command), 
        .shift(calibtrain));
    lo_calibPulseTrain calibPulseTrain(.out(calibTrain), .clk(clk),
        .clrB(int_clrB), .oneImp(~calibParams[1]), 
        .only100Imps(~calibParams[0]), .start(calibTrainStart));

    //Read data sequence
    lo_dataReadout dataRoeadout(.shiftClkEnable(shiftClkEnable),
        .readHeader(readHeader), .tokenStart(tokenStart), .tokenClk(tokenClk),
        .clk(clk), .clrB(int_clrB), .start(readdata));

    //Command decoder with delays
    wire initializeNoDel, loadandclearNoDel, loadramNoDel, readdataNoDel,
        invalidaddrNoDel, counterspulseNoDel, setgateNoDel, calibtrainNoDel,
        loaddacNoDel;
    assign #1 initialize=initializeNoDel;
    assign #1 loadandclear=loadandclearNoDel;
    assign #2 loadram=loadramNoDel;
    assign #3 readdata=readdataNoDel;
    assign #1 invalidaddr=invalidaddrNoDel;
    assign #1 counterspulse=counterspulseNoDel;
    assign #2 setgate=setgateNoDel;
    assign #2 calibtrain=calibtrainNoDel;
    assign #1 loaddac=loaddacNoDel;

    lo_commandDecoder commandDecoder(.Initialize(initializeNoDel),
        .LoadAndClear(loadandclearNoDel), .LoadRam(loadramNoDel),
        .ReadData(readdataNoDel), .InvalidAddr(invalidaddrNoDel),
        .CountersPulse(counterspulseNoDel), .SetGate(setgateNoDel),
        .CalibTrain(calibtrainNoDel), .LoadDac(loaddacNoDel),
        .id0(id[0]), .id1(id[1]), 
        .command(command), .clk(clk), .clrB(int_clrB));
endmodule

`timescale 1ns/1ps

module  lo_12shiftRegAddr(data, strobe, clk, clrB, in, shift);

    output [7:0] data;
    output [11:0] strobe;
    input clk, clrB, in, shift;

    reg [7:0] data;
    reg [3:0] address;

    wire oneToZero, sendStrobe;

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
        begin
		    data <= 8'd0;		
		    address <= 4'd0;		
        end else 
            if (shift) begin
                data[7:0] <= {data[6:0],in};
                address[3:0]<={address[2:0],data[7]};
            end else
            begin
                data[7:0] <= data;
                address[3:0] <= address;
            end
    end
    lo_catch1to0tran catchMod(.out(oneToZero), .clk(clk), .clrB(clrB), .in(shift));
    lo_3clkImp longImp(.out(sendStrobe), .clk(clk), .clrB(clrB), .start(oneToZero));

    assign #1 strobe[0]  = ~address[0] & ~address[1] & ~address[2] & ~address[3] & sendStrobe;
    assign #2 strobe[1]  =  address[0] & ~address[1] & ~address[2] & ~address[3] & sendStrobe;
    assign #1 strobe[2]  = ~address[0] &  address[1] & ~address[2] & ~address[3] & sendStrobe;
    assign #3 strobe[3]  =  address[0] &  address[1] & ~address[2] & ~address[3] & sendStrobe;
    assign #2 strobe[4]  = ~address[0] & ~address[1] &  address[2] & ~address[3] & sendStrobe;
    assign #1 strobe[5]  =  address[0] & ~address[1] &  address[2] & ~address[3] & sendStrobe;
    assign #1 strobe[6]  = ~address[0] &  address[1] &  address[2] & ~address[3] & sendStrobe;
    assign #4 strobe[7]  =  address[0] &  address[1] &  address[2] & ~address[3] & sendStrobe;
    assign #1 strobe[8]  = ~address[0] & ~address[1] & ~address[2] &  address[3] & sendStrobe;
    assign #3 strobe[9]  =  address[0] & ~address[1] & ~address[2] &  address[3] & sendStrobe;
    assign #4 strobe[10] = ~address[0] &  address[1] & ~address[2] &  address[3] & sendStrobe;
    assign #4 strobe[11] =  address[0] &  address[1] & ~address[2] &  address[3] & sendStrobe;
endmodule
`timescale 1ns/1ps

module  lo_2shiftRegMem(out, clk, clrB, in, shift);

    output [1:0] out;
    input clk, clrB, in, shift;

    wire [1:0] out;
    reg [1:0] data, mem;

    assign #3 out = mem;

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
        begin
		    data <= 2'b00;		
		    mem <= 2'b00;		
        end else 
            if (shift) begin
                data[1:0] <= {data[0],in};
                mem<=mem;
            end else
            begin
                data[1:0] <= data;
                mem <= data;
            end
    end
endmodule
`timescale 1ns/1ps

module  lo_2shiftRegStrobe(out, finish, clk, clrB, in, shift);

    output [1:0] out;
    output finish;
    input clk, clrB, in, shift;

    reg [1:0] data;
    assign #1 out = data;

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
        begin
		    data <= 2'b00;		
        end else 
            if (shift) begin
                data[1:0] <= {data[0],in};
            end else
            begin
                data[1:0] <= data;
            end
    end
    lo_catch1to0tran catchMod(.out(finish), .clk(clk), .clrB(clrB), .in(shift));
endmodule
`timescale 1ns/1ps

module lo_3clkImp(out, clk, clrB, start);

    output out;
    input clk, clrB, start;

    reg [0:1] state, next;

    assign #6 out = state[0] | state[1];

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
		    state <= 2'b00;		
	    else state<= next;
    end
   
    always @(state or start)
    begin
        case (state)
            2'b00: if (start==0) next = 2'b00;
                   else next = 2'b01;     
            2'b01: next = 2'b10;     
            2'b10: next = 2'b11;     
            2'b11: next = 2'b00;     
        endcase
    end
endmodule
`timescale 1ns/1ps

module  lo_calibPulseTrain(out, clk, clrB, oneImp, only100Imps, start);

    output out;
    input clk, clrB, oneImp, only100Imps, start;

    reg loPulse;

    wire startImp, stop, already100, already500, nextImp, pulseEnd, pulseOut;
    wire [9:0] counterValue;

    assign #1 startImp = start | (nextImp & ~stop);
    assign #1 out = ~loPulse & pulseOut;

    assign #2 stop = oneImp | (already100 & only100Imps) | (already500 & ~only100Imps);
    assign #4 already100 = (counterValue== 10'd200) ? 1'b1 : 1'b0;
    assign #5 already500 = (counterValue== 10'd1000) ? 1'b1 : 1'b0;
    //assign already100 = (counterValue>= 10'd10) ? 1'b1 : 1'b0;
    //assign already500 = (counterValue>= 10'd50) ? 1'b1 : 1'b0;

    lo_254clkImp x1(.out(pulseOut), .finish(pulseEnd), .clk(clk),
        .clrB(clrB), .start(startImp));
    
    lo_parCounter count(.value(counterValue), .clk(clk), .clrB(clrB),
        .synchClr(start), .pulse(pulseEnd));
    defparam count.width = 10;        // parameter redefinition
    
    lo_del1clk delay(.out(nextImp), .clk(clk), .clrB(clrB), .in(pulseEnd));

    always @(posedge clk or negedge clrB)
    begin
        if (!clrB) begin
            loPulse <= 1'b0;
        end 
        else begin
            loPulse <= ~start & loPulse^pulseEnd;
        end
    end
endmodule
`timescale 1ns/1ps

module lo_commandDecoder(Initialize, LoadAndClear, LoadRam, ReadData, InvalidAddr, 
	CountersPulse, SetGate, CalibTrain, LoadDac, id0, id1, command, clk, clrB);
output Initialize, LoadAndClear, LoadRam, ReadData, InvalidAddr, CountersPulse, SetGate, CalibTrain, LoadDac;
input clk, clrB, id0, id1, command;
parameter [5:0] //synopsys enum state_info
	HEADER_OK = 6'h0,
	H1 = 6'h1,
	START = 6'h2,
	H2 = 6'h3,
	LD10 = 6'h4,
	WAIT3 = 6'h5,
	WAIT2 = 6'h6,
	CP1 = 6'h7,
	LD4 = 6'h8,
	LD1 = 6'h9,
	NOP6 = 6'ha,
	ADDR1_OK = 6'hb,
	LD2 = 6'hc,
	NOP4 = 6'hd,
	LD6 = 6'he,
	LD7 = 6'hf,
	LD5 = 6'h10,
	WAIT1 = 6'h11,
	WAIT0 = 6'h12,
	NOP15 = 6'h13,
	WAIT6 = 6'h14,
	WAIT5 = 6'h15,
	WAIT4 = 6'h16,
	NOP13 = 6'h17,
	NOP12 = 6'h18,
	NOP11 = 6'h19,
	NOP10 = 6'h1a,
	NOP9 = 6'h1b,
	NOP8 = 6'h1c,
	NOP7 = 6'h1d,
	LD9 = 6'h1e,
	LD8 = 6'h1f,
	NOP14 = 6'h20,
	NOP3 = 6'h21,
	NOP2 = 6'h22,
	LD3 = 6'h23,
	CMD110 = 6'h24,
	CMD_001 = 6'h25,
	CMD_01x = 6'h26,
	CMD_101 = 6'h27,
	CMD_11x = 6'h28,
	NOP1 = 6'h29,
	CMD_011 = 6'h2a,
	CMD_0xx = 6'h2b,
	CMD_010 = 6'h2c,
	CMD_100 = 6'h2d,
	CMD_111 = 6'h2e,
	NOP0 = 6'h2f,
	CMD_1xx = 6'h30,
	CMD_000 = 6'h31,
	Global1 = 6'h32,
	ADDR_OK = 6'h33,
	CMD_00x = 6'h34,
	LOCAL_ADDR = 6'h35,
	CMD_10x = 6'h36,
	GLOBAL_ADDR = 6'h37,
	H3 = 6'h38,
	SG1 = 6'h39,
	NOP5 = 6'h3a,
	LD11 = 6'h3b,
	WAIT7 = 6'h3c;

reg [5:0] /* synopsys enum state_info */ state;
reg [5:0] /* synopsys enum state_info */ next;
reg Initialize, LoadAndClear, LoadRam, ReadData, InvalidAddr, CountersPulse, SetGate, CalibTrain, LoadDac;

//synopsys state_vector state 

always @(posedge clk or negedge clrB) begin
  if(!clrB) state <= START;
  else state <= next;
end
always @(id0 or id1 or command or state)
begin
// next state definitions

  case(state)//synopsys full_case parallel_case
    WAIT7:
        begin
          next = NOP13;
        end
    LD11:
        begin
          next = START;
        end
    NOP5:
        begin
          next = NOP6;
        end
    SG1:
        begin
          next = NOP6;
        end
    H3:
        begin
          if(command==1)
            next = START;
          else
            next = HEADER_OK;
        end
    GLOBAL_ADDR:
        begin
          if(command==1)
            next = NOP0;
          else
            next = Global1;
        end
    CMD_10x:
        begin
          if(command==1)
            next = CMD_101;
          else
            next = CMD_100;
        end
    LOCAL_ADDR:
        begin
          if(command==1)
            if(id1==1)
              next = ADDR1_OK;
            else
              next = NOP0;
          else
            if(id1==1)
              next = NOP0;
            else
              next = ADDR1_OK;
        end
    CMD_00x:
        begin
          if(command==1)
            next = CMD_001;
          else
            next = CMD_000;
        end
    ADDR_OK:
        begin
          if(command==1)
            next = CMD_1xx;
          else
            next = CMD_0xx;
        end
    Global1:
        begin
          if(command==1)
            next = NOP1;
          else
            next = ADDR_OK;
        end
    CMD_000:
        begin
          next = SG1;
        end
    CMD_1xx:
        begin
          if(command==1)
            next = CMD_11x;
          else
            next = CMD_10x;
        end
    NOP0:
        begin
          next = NOP1;
        end
    CMD_111:
        begin
          next = NOP5;
        end
    CMD_100:
        begin
          next = CP1;
        end
    CMD_010:
        begin
          next = NOP5;
        end
    CMD_0xx:
        begin
          if(command==1)
            next = CMD_01x;
          else
            next = CMD_00x;
        end
    CMD_011:
        begin
          next = NOP5;
        end
    NOP1:
        begin
          next = NOP2;
        end
    CMD_11x:
        begin
          if(command==1)
            next = CMD_111;
          else
            next = CMD110;
        end
    CMD_101:
        begin
          next = NOP5;
        end
    CMD_01x:
        begin
          if(command==1)
            next = CMD_011;
          else
            next = CMD_010;
        end
    CMD_001:
        begin
          next = WAIT0;
        end
    CMD110:
        begin
          next = LD1;
        end
    LD3:
        begin
          next = LD4;
        end
    NOP2:
        begin
          next = NOP3;
        end
    NOP3:
        begin
          next = NOP4;
        end
    NOP14:
        begin
          next = NOP15;
        end
    LD8:
        begin
          next = LD9;
        end
    LD9:
        begin
          next = LD10;
        end
    NOP7:
        begin
          next = NOP8;
        end
    NOP8:
        begin
          next = NOP9;
        end
    NOP9:
        begin
          next = NOP10;
        end
    NOP10:
        begin
          next = NOP11;
        end
    NOP11:
        begin
          next = NOP12;
        end
    NOP12:
        begin
          next = NOP13;
        end
    NOP13:
        begin
          next = NOP14;
        end
    WAIT4:
        begin
          next = WAIT5;
        end
    WAIT5:
        begin
          next = WAIT6;
        end
    WAIT6:
        begin
          next = WAIT7;
        end
    NOP15:
        begin
          next = START;
        end
    WAIT0:
        begin
          next = WAIT1;
        end
    WAIT1:
        begin
          next = WAIT2;
        end
    LD5:
        begin
          next = LD6;
        end
    LD7:
        begin
          next = LD8;
        end
    LD6:
        begin
          next = LD7;
        end
    NOP4:
        begin
          next = NOP5;
        end
    LD2:
        begin
          next = LD3;
        end
    ADDR1_OK:
        begin
          if(command==1)
            if(id0==1)
              next = ADDR_OK;
            else
              next = NOP1;
          else
            if(id0==1)
              next = NOP1;
            else
              next = ADDR_OK;
        end
    NOP6:
        begin
          next = NOP7;
        end
    LD1:
        begin
          next = LD2;
        end
    LD4:
        begin
          next = LD5;
        end
    CP1:
        begin
          next = NOP6;
        end
    WAIT2:
        begin
          next = WAIT3;
        end
    WAIT3:
        begin
          next = WAIT4;
        end
    LD10:
        begin
          next = LD11;
        end
    H2:
        begin
          if(command==1)
            next = H3;
          else
            next = START;
        end
    START:
        begin
          if(command==1)
            next = H1;
          else
            next = START;
        end
    H1:
        begin
          if(command==1)
            next = START;
          else
            next = H2;
        end
    HEADER_OK:
        begin
          if(command==1)
            next = GLOBAL_ADDR;
          else
            next = LOCAL_ADDR;
        end
    default: next = START;
  endcase

// Conditional output definitions

if(state==CMD_010) Initialize = 1;
  else Initialize = 0;

if(state==CMD_111) LoadAndClear = 1;
  else LoadAndClear = 0;

if(state==CMD_011) LoadRam = 1;
  else LoadRam = 0;

if(state==WAIT7) ReadData = 1;
  else ReadData = 0;

if(state==NOP4) InvalidAddr = 1;
  else InvalidAddr = 0;

if(state==CMD_101) CountersPulse = 1;
  else CountersPulse = 0;

if(state==SG1
  || state==CMD_000) SetGate = 1;
  else SetGate = 0;

if(state==CMD_100
  || state==CP1) CalibTrain = 1;
  else CalibTrain = 0;

if(state==LD7
  || state==LD6
  || state==LD5
  || state==LD4
  || state==LD9
  || state==LD8
  || state==CMD110
  || state==LD1
  || state==LD2
  || state==LD3
  || state==LD10
  || state==LD11) LoadDac = 1;
  else LoadDac = 0;

end
endmodule
`timescale 1ns/1ps

module lo_dataReadout(shiftClkEnable, readHeader, tokenStart, tokenClk,
    clk, clrB, start);

    output shiftClkEnable, readHeader, tokenStart, tokenClk;
    input clk, clrB, start;

    wire startShiftClk, stopShiftClk, nextToken, stop;

    wire [7:0] tokenNumber;
    wire [3:0] readPulseNumber;
    wire tokens96, readPulses10;

    wire tokenClkNoDel, startShiftClkNoDel, stopShiftClkNoDel, readHeaderNoDel, 
        tokenStartNoDel, tokenStartStart;
    //Potrzebne tylko zeby calosc byla bardziej realistyczna bo w module
    // lo_readDataSeq nie ma zadych opoznien
    assign #3 tokenClk = tokenClkNoDel;
    assign #2 startShiftClk = startShiftClkNoDel;
    assign #4 stopShiftClk = stopShiftClkNoDel;
    assign #3 readHeader = readHeaderNoDel;
    assign #1 tokenStartStart = tokenStartNoDel;

    assign #4 tokens96 = (tokenNumber>= 8'd96) ? 1'b1 : 1'b0;
    assign #5 readPulses10 = (readPulseNumber >= 4'd8) ? 1'b1 : 1'b0;

    lo_readDataSeq dataSeq(.tokenClk(tokenClkNoDel), 
        .startShiftClk(startShiftClkNoDel),
        .stopShiftClk(stopShiftClkNoDel), .readHeader(readHeaderNoDel), 
        .tokenStart(tokenStartNoDel), .start(start), .nextToken(readPulses10),
        .stop(tokens96), .clk(clk), .clrB(clrB));
    
    lo_3clkImp readHeaderImpMod(.out(tokenStart), .clk(clk), .clrB(clrB),
        .start(tokenStartStart));
    
    lo_rs shiftClkRS(.out(shiftClkEnable), .clk(clk), .clrB(clrB),
        .set(startShiftClk), .reset(stopShiftClk));

    lo_parCounter tokenCounter(.value(tokenNumber), .clk(clk), .clrB(clrB),
        .synchClr(readHeader), .pulse(tokenClk));
    defparam tokenCounter.width = 8;        // parameter redefinition

    lo_parCounter readPulseCounter(.value(readPulseNumber), .clk(clk),
        .clrB(clrB), .synchClr(tokenClk), .pulse(shiftClkEnable));
    defparam readPulseCounter.width = 4;        // parameter redefinition

endmodule
`timescale 1ns/1ps

module  lo_del1clk(out, clk, clrB, in);

    output out;
    input clk, clrB, in;

    reg state;
    assign #4 out = state;

    always @(posedge clk or negedge clrB)
        if (!clrB) state <= 1'b0;
        else state<= in;
endmodule
`timescale 1ns/1ps

module  lo_loadOrClear(go, finish, disableGates, clk, clrB, start);

    output go, finish, disableGates;
    input clk, clrB, start;

    wire startImp, waitToEnd, delWaitToEnd;

    lo_rs rs(.out(disableGates), .clk(clk), .clrB(clrB), .set(start), .reset(delWaitToEnd));
    lo_wait3clk wait1(.out(startImp), .clk(clk), .clrB(clrB), .start(start));
    lo_3clkImp imp(.out(go), .clk(clk), .clrB(clrB), .start(startImp));
    lo_catch1to0tran catch(.out(waitToEnd), .clk(clk), .clrB(clrB), .in(go));
    lo_del1clk delay1(.out(delWaitToEnd), .clk(clk), .clrB(clrB), .in(waitToEnd));
    lo_del1clk delay2(.out(finish), .clk(clk), .clrB(clrB), .in(delWaitToEnd));
endmodule
`timescale 1ns/1ps

module  lo_parMem(out, clk, clrB, strobe, in);
    
    //default value. Should be defined in upper level module
    parameter width = 4;

    output [width-1:0] out;
    input clk, clrB, strobe;
    input [width-1:0] in;

    wire [width-1:0] out;
    reg [width-1:0] mem;

    assign #1 out = mem;

    always @(posedge clk or negedge clrB)
        if (!clrB) mem <= 0;
        else if (strobe)
            mem <= in;
endmodule
`timescale 1ns/1ps

module lo_rs(out, clk, clrB, set, reset);

    output out;
    input clk, clrB, set, reset;

    reg state, next;
    assign #3 out = state;

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) state <= 1'b0;		
        else state <= next;
    end

    always @(state or set or reset)
        case ({state,set,reset})
            3'b000: next=1'b0;
            3'b001: next=1'b0;
            3'b010: next=1'b1;
            3'b011: next=1'b0;
            3'b100: next=1'b1;
            3'b101: next=1'b0;
            3'b110: next=1'b1;
            3'b111: next=1'b0;
        endcase
endmodule
`timescale 1ns/1ps

module  lo_254clkImp(out, finish, clk, clrB, start);

    output out, finish;
    input clk, clrB, start;

    reg [8:0] q;

    wire x;

    assign #2 out = | q[7:0];
    assign #2 finish = ~out & q[8];
    assign #3 x = ~(q[7]^q[3]^q[2]^q[1]) & (out | start);

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) q <= 0;		
        else q[8:0] <= {q[7:0],x};
    end
endmodule
`timescale 1ns/1ps

module lo_catch1to0tran(out, clk, clrB, in);

    output out;
    input clk, clrB, in;

    //reg out;
    reg [0:1] state, next;

    assign #5 out = state[0] & ~state[1]; //out = 1 if state == 10

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
		    state <= 2'b00;		
	    else state<= next;
    end
   
    always @(state or in)
    begin
        case (state)
            2'b00: if (in==0) next = 2'b00;
                   else next = 2'b01;     
            2'b01: if (in==0) next = 2'b10;
                   else next = 2'b01;     
            2'b10: next = 2'b00;
            default: next = 2'b00;     
        endcase
        //if (state == 2'b10) out = 1'b1;
        //else out = 1'b0;
    end
endmodule
`timescale 1ns/1ps

module lo_parCounter(value, clk, clrB, synchClr, pulse);
    
    //default value. Should be defined in upper level module
    parameter width = 4;
	
    output value;
	input clk, clrB, synchClr, pulse;

	reg [width-1:0] value;


	always @(posedge clk or negedge clrB)
		if (!clrB) value <= 0;		
		else if (synchClr) 
            value <= 0;
        else if (pulse) 
			value <= value + 1;
endmodule
`timescale 1ns/1ps

module lo_readDataSeq(tokenClk, startShiftClk, stopShiftClk, readHeader, tokenStart, start, 
	nextToken, stop, clk, clrB);
output tokenClk, startShiftClk, stopShiftClk, readHeader, tokenStart;
input clk, clrB, start, nextToken, stop;
parameter [3:0] //synopsys enum state_info
	SC0 = 4'h0,
	H0 = 4'h1,
	H1 = 4'h2,
	START = 4'h3,
	H2 = 4'h4,
	H3 = 4'h5,
	H4 = 4'h6,
	H5 = 4'h7,
	CDA = 4'h8;

reg [3:0] /* synopsys enum state_info */ state;
reg [3:0] /* synopsys enum state_info */ next;
reg tokenClk, startShiftClk, stopShiftClk, readHeader, tokenStart;

//synopsys state_vector state 

always @(posedge clk or negedge clrB) begin
  if(!clrB) state <= START;
  else state <= next;
end
always @(start or nextToken or stop or state)
begin
// next state definitions

  case(state)//synopsys full_case parallel_case
    CDA:
        begin
          if(nextToken==1)
            if(stop==1)
              next = START;
            else
              next = H5;
          else
            next = CDA;
        end
    H5:
        begin
          next = CDA;
        end
    H4:
        begin
          next = H5;
        end
    H3:
        begin
          next = H4;
        end
    H2:
        begin
          next = H3;
        end
    START:
        begin
          if(start==1)
            next = SC0;
          else
            next = START;
        end
    H1:
        begin
          next = H2;
        end
    H0:
        begin
          next = H1;
        end
    SC0:
        begin
          next = H0;
        end
    default: next = START;
  endcase

// Conditional output definitions

if(state==START) stopShiftClk = 1;
  else stopShiftClk = 0;

if(state==H5) tokenClk = 1;
  else tokenClk = 0;

if(state==SC0) startShiftClk = 1;
  else startShiftClk = 0;

if(state==H0) readHeader = 1;
  else readHeader = 0;

if(state==H3) tokenStart = 1;
  else tokenStart = 0;

end
endmodule
`timescale 1ns/1ps

module lo_wait3clk(out, clk, clrB, start);

    output out;
    input clk, clrB, start;

    //reg out;
    reg [0:1] state, next;

    assign #3 out = state[0] & state[1];

    always @(posedge clk or negedge clrB)
    begin
	    if (!clrB) 
		    state <= 2'b00;		
	    else state <= next;
    end
   
    always @(state or start)
    begin
        case (state)
            2'b00: if (start==0) next = 2'b00;
                   else next = 2'b01;     
            2'b01: next = 2'b10;     
            2'b10: next = 2'b11;     
            2'b11: next = 2'b00;     
        endcase
        //if (state == 2'b11) out = 1'b1;
        //else out = 1'b0;
    end
endmodule
`timescale 1ns/1ps

module  lo_4clkReset(out, clk, in);

    output out;
    input clk, in;

    reg [3:0] data;
    assign #3 out = | data[3:0];

    always @(posedge clk)
        data[3:0] <= {data[2:0],in};
endmodule
